package reti;

/**
 * Title:        Progetto di reti di calcolatori
 * Description:  Sistema di server commerciali per la vendita di prodotti via internet
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Alessandro Celli
 * @version 1.0
 */
import java.util.ArrayList;
import java.util.Hashtable;

/*****************************  CLASSI ASTRATTE DI BASE ********************************/
public abstract class Msg {
  public Msg() {};
  short opCode;
  String toMsg(){ return Short.toString(opCode)+"\n"; };
  static short getOpCode(String in){ return Short.parseShort(in.substring(0,in.indexOf(''))); };
  short getopCode() { return opCode; };
}

/** Acknowledge generico, non fornisce nessuna informazione.
 *  Il suo scopo  solo quello di creare dei punti di sincronizzazione nelle comunicazioni */
class genericAckMsg extends Msg {
  genericAckMsg() { opCode = stringCommand.GENERIC_ACK_CODE; }
  String toMsg() { return Short.toString(opCode)+"\n"; };
  short getopCode() { return opCode; };
};

/************************  CLASSI PER LA GESTIONE DEI CLIENT ***************************/
abstract class cliMsg extends Msg {
  public cliMsg() {};
}

/** Pacchetto inviato dal client subito dopo la connessione al server.
 *  Consente di informare il server dello stato del client */
class cliConnectMsg extends cliMsg {
  long CL; // Clock Logico del sistema
  String cliCode; // Codice del cliente
  cliConnectMsg() { opCode = stringCommand.CLI_CONNECT_CODE; CL = 0; cliCode = ""; };
  cliConnectMsg(long cl,String clicode) {
    opCode = stringCommand.CLI_CONNECT_CODE;
    CL = cl;
    cliCode = clicode;
  };
  cliConnectMsg(String in) {
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    CL = Integer.parseInt(in.substring(0,pos)); // Estraggo il Clock Logico
    in = in.substring(pos+1);
    pos = in.indexOf('');
    cliCode = in.substring(0,pos);            // Estraggo il codice del cliente
  };
  String toMsg(){ return Short.toString(opCode)+""+Long.toString(CL)+""+cliCode+"\n"; };
  String getCliCode() { return cliCode; };  // Restituisce il codice del cliente
  short getOpCode() { return opCode; };     // Restituisce il codice della operazione
  long getCl() { return CL; };              // Restituisce il valore del Clock Logico
}
/** Acknowledge alla connessione,  usata dal server per fornire al client l'accettazione e le
 *  informazioni sullo stato attuale del sistema. Qualora queste siano diverse dalle attuali il
 *  Server forza il client ad una rilettura dello stato attraverso gli appositi pacchetti */
class cliAckMsg extends cliMsg {
  long CL;        // Clock Logico del sistema
  String cliCode; // Codice del cliente
  ArrayList info; // Informazioni per gli utenti
  cliAckMsg(){ opCode = stringCommand.CLI_ACK_CODE;  CL = 0;  cliCode = ""; info = new ArrayList(); };
  cliAckMsg(long cl,String clicode) {
    opCode = stringCommand.CLI_ACK_CODE;
    CL = cl;
    cliCode = clicode;
    info = new ArrayList();
  };
  cliAckMsg(long cl,String clicode,ArrayList inf) {
    opCode = stringCommand.CLI_ACK_CODE;
    CL = cl;
    cliCode = clicode;
    if (inf==null)info = new ArrayList();
    else info = inf;
  };
  cliAckMsg(String in) {
    info = new ArrayList();
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    CL = Integer.parseInt(in.substring(0,pos));     // Estraggo il Clock Logico
    in = in.substring(pos+1);
    pos = in.indexOf('');
    cliCode = in.substring(0,pos);                 // Estraggo il Server Code
    in = in.substring(pos+1);
    while (!(in.compareTo("") ==0)){               // Se ho esaurito l'input
      pos = in.indexOf('');
      info.add(in.substring(0,pos));   // Estraggo le info
      in = in.substring(pos+1);
    };
  };
  String toMsg(){
    String app = Short.toString(opCode)+""+Long.toString(CL)+""+cliCode+"";
    if (info!=null) {
      for (int i=0;i<info.size();i++){ app = app+((String)info.get(i))+""; };
    };
    return app+"\n";
  };
  String getCliCode() { return cliCode; };  // Restituisce il codice del cliente
  short getOpCode() { return opCode; };     // Restituisce il codice della operazione
  long getCl() { return CL; };              // Restituisce il valore del Clock Logico
//  void addInfo(String i1,String i2) { info.add(new msgInfo(i1,i2)); }; // Aggiunge una info alla lista delle informazioni
  void addInfo(String i) { info.add(i); }; // Aggiunge una info alla lista delle informazioni
  int getNumInfo() { return info.size(); };
  String getInfo(int pos) { return ((String)info.get(pos)); };
}
class cliListProdReqMsg extends cliMsg {
  cliListProdReqMsg() { opCode = stringCommand.CLI_LIST_PROD_REQ_CODE; };
  cliListProdReqMsg(String in) {
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
  };
  String toMsg(){ return Short.toString(opCode)+"\n";};
  short getOpCode() { return opCode; };     // Restituisce il codice della operazione

}
class cliListProdReplyMsg extends cliMsg {
  ArrayList product; // Lista dei prodotti disponibili
  cliListProdReplyMsg() {
    opCode = stringCommand.CLI_LIST_PROD_REPLY_CODE;
    product = new ArrayList();
  };
  cliListProdReplyMsg(String in) {
    int pos;
    product = new ArrayList();
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    while (!in.equalsIgnoreCase ("")) {
      pos = in.indexOf('');
      product.add(in.substring(0,pos));
      in = in.substring(pos+1);
    };
  };
  String toMsg(){
    String app="";
    app = Short.toString(opCode) + "";
    while (!product.isEmpty()){
      app = app + product.get(0) + "";
      product.remove(0);
    }
    return app+"\n";};
  short getOpCode() { return opCode; };         // Restituisce il codice della operazione
  int getNumProd() { return product.size(); };  // Restituisce il numero di prodotti elencati
  String getProd(int i) { return ((String)product.get(i)); }; // Restituisce il prodotto i-esimo
  void addProd(String p) { product.add(p); };   // Aggiunge un prodotto alla lista
  void setProd(ArrayList al) { product=al; };   // Imposta la lista dei prodotti
}

/**Si incarica di contenere tutte le informazioni necessarie per consentire al server di registare un ordine*/
class cliSendOrderMsg extends cliMsg {
  String cliCode;
  ArrayList product;
  cliSendOrderMsg() {
    opCode = stringCommand.CLI_SEND_ORDER_CODE;
    product = new ArrayList(); // Lista dei prodotti richiesti
  };
  cliSendOrderMsg(String clicode,ArrayList prod) {
    opCode = stringCommand.CLI_SEND_ORDER_CODE;
    if (prod==null) product = new ArrayList();
    else product = prod; // Lista dei prodotti richiesti
    cliCode = clicode;
  };
  cliSendOrderMsg(String in) {//  RICEVUTO CLI:8a00001sv01=1mb01=1
    product = new ArrayList(); // Lista dei prodotti richiesti
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    cliCode = in.substring(0,pos);              // Estraggo il codice del cliente
    in = in.substring(pos+1);
    while (!(in.compareTo("") ==0)){               // Se ho esaurito l'input
      pos = in.indexOf('');
      product.add(new msgInfo(in.substring(0,pos)));   // Estraggo le info
      in = in.substring(pos+1);
    };
  };
  String toMsg(){
    String app="";
    app = Short.toString(opCode) + "" + cliCode + "";
    if (product!=null) {
      for (int i=0;i<product.size();i++){ app = app+((msgInfo)product.get(i)).toMsg()+""; };
    };
    return app+"\n";
  };
  String getCliCode() { return cliCode; };  // Restituisce il codice del cliente
  short getOpCode() { return opCode; };     // Restituisce il codice della operazione
  void addInfo(String i1,String i2) { product.add(new msgInfo(i1,i2)); }; // Aggiunge una info alla lista delle informazioni
  void addInfo(String i) { product.add(new msgInfo(i)); }; // Aggiunge una info alla lista delle informazioni
  void addInfo(String i,char s) { product.add(new msgInfo(i,s)); }; // Aggiunge una info alla lista delle informazioni
  msgInfo getInfo(int pos) { return ((msgInfo) product.get(pos)); };
  int getNumInfo() { return product.size(); };
  ArrayList getAllInfo() { return product; };
}

class cliAckOrderMsg extends cliMsg {
  String orderCode; // Codice dell'ordine effettuato
  String servCode;  // Codice del server che conferma l'ordine
  String cliCode;   // Codice del cliente che ha inviato l'ordine
  int result;       // Risultato della transazione effettuata
  // *********** COSTANTI PER LA GESTIONE DELLE RISPOSTE ************
  public static final short ORDER_ACCEPT  =  0;
  public static final short END_PRODUCT   = -1;
  public static final short SOCKET_BROKE  = -2;
  public static final short ERROR_ON_CODE = -3;
  public static final short UNKNOW_ERROR  = -4;
  public static final String NO_ORDER_CODE= "NoOrder";
  cliAckOrderMsg() {
    opCode = stringCommand.CLI_ACK_ORDER_CODE;
    orderCode = "";
    servCode = "";
    cliCode = "";
    result = UNKNOW_ERROR;
  }
  cliAckOrderMsg(String ordercode,String servcode,String clicode, int res) {
    opCode = stringCommand.CLI_ACK_ORDER_CODE;
    orderCode = ordercode;
    servCode = servcode;
    cliCode = clicode;
    result = res;
  }
  String toMsg(){
    return Short.toString(opCode)+""+orderCode+""+servCode+""+cliCode+""+Integer.toString(result)+"\n";
  };
  cliAckOrderMsg(String in) {
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    orderCode = in.substring(0,pos);              // Estraggo il codice del ordine
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);              // Estraggo il codice del server che ha ricevuto l'ordine
    in = in.substring(pos+1);
    pos = in.indexOf('');
    cliCode = in.substring(0,pos);              // Estraggo il codice del cliente che ha inviato l'ordine
    in = in.substring(pos+1);
    pos = in.indexOf('');
    result = Integer.parseInt(in.substring(0,pos)); // Estraggo il risultato dell'operazione
  };
  String getOrderCode(){ return orderCode;};  // Restituisce il codice del ordine
  String getServCode() { return servCode; };  // Restituisce il codice del server
  String getCliCode()  { return cliCode;  };  // Restituisce il codice del cliente
  int getResult() { return result; };     // Restituisce il risultato della operazione
  void setOrderCode(String ordercode){ orderCode = ordercode; };// Restituisce il codice del ordine
  void setServCode(String servcode)  { servCode = servcode; };  // Restituisce il codice del server
  void setCliCode(String clicode)    { cliCode = clicode; };    // Restituisce il codice del cliente
  void setResult(int res) { result = res; };  // Restituisce il risultato
};

class cliAcceptNewServerMsg extends cliMsg {
  String servCode; // Codice del server che effettua la richiesta di connessione
  String ip;
  int myPort;
  int port;
  cliAcceptNewServerMsg() { opCode = stringCommand.CLI_ACCEPT_NEW_SERVER_CODE; servCode = ""; port=0; ip="127.0.0.1"; };
  cliAcceptNewServerMsg(short opcode, String servcode, String Ip,int Port) {
    opCode = stringCommand.CLI_CONNECT_CODE;
    servCode = servcode;
    ip = Ip;
    port = Port;
  };
  cliAcceptNewServerMsg(String in) {
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);            // Estraggo il codice del cliente
    in = in.substring(pos+1);
    pos = in.indexOf('');
    ip = in.substring(0,pos);            // Estraggo l'ip del nuovo server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    port = Integer.parseInt(in.substring(0,pos)); // Estraggo la porta per il previous server
  };
  String toMsg(){ return Short.toString(opCode)+""+servCode+""+ip+""+Integer.toString(port)+"\n"; };
  String getServCode() { return servCode; };       // Restituisce il codice del server
  void setServCode(String sc) { servCode = sc; };  // Imposta il codice del server
  short getOpCode() { return opCode; };       // Restituisce il codice della operazione
  int getPort() { return port; };
  void setPort(int p) { port = p; };
  String getIp() { return ip; };
  void setIp(String IP) { ip = IP; };
};

class cliServAckMsg extends cliMsg {
  String servCode; // Codice del server
  long CL;
  boolean accept;  // Specifica se la connessione  stata accettata
  ArrayList info; // Lista dei server attivi
  cliServAckMsg(){
    opCode = stringCommand.CLI_SERV_ACK_CODE;
    CL = 0;
    servCode = "";
    accept=false;
    info = new ArrayList();
  };
  cliServAckMsg(boolean accepted, String servcode, long cl, ArrayList inf) {
    opCode = stringCommand.CLI_SERV_ACK_CODE;
    accept = accepted;
    servCode = servcode;
    CL = cl;
    if (inf==null) info = new ArrayList();
    else info = inf;
  };
  cliServAckMsg(String in) {
    info = new ArrayList();
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    String app = in.substring(0,pos);
    if (app.compareToIgnoreCase("true")==0) accept = true;
    else accept = false;
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);     // Estraggo il server Code
    in = in.substring(pos+1);
    pos = in.indexOf('');
    CL = Long.parseLong(in.substring(0,pos));     // Estraggo il clock Logico
    in = in.substring(pos+1);
    while (!(in.compareTo("") ==0)){               // Se ho esaurito l'input
      pos = in.indexOf('');
      info.add(in.substring(0,pos));      // Estraggo i codici dei server e li aggiungo alle info
      in = in.substring(pos+1);
    };
  };
  String toMsg(){
    String app = Short.toString(opCode)+"";
    if (accept) app = app + "true";
    else app = app + "false";
    app = app+servCode+""+Long.toString(CL)+"";
    for (int i=0;i<info.size();i++) app = app + (String)info.get(i) + "";
    return app+"\n";
  };
  String getServCode() { return servCode; };  // Restituisce il codice del servitore
  short getOpCode() { return opCode; };     // Restituisce il codice della operazione
  boolean isAccept() { return accept; };
  void   setAccept(boolean acc){ accept = acc; };
  long getCl() { return CL; };
  void setCl(long cl){ CL = cl; };
  void addInfo(String inf) { info.add(inf); }; // Aggiunge un nome alla lista dei server
  String getInfo(int pos) { return ((String)info.get(pos)); };
  int getNumInfo() { return info.size(); };
  ArrayList getAllInfo() { return info; };
}

class cliDelOrderMsg extends cliMsg {
  String orderCode;
  String cliCode;
  cliDelOrderMsg() {
    opCode = stringCommand.CLI_DELETE_ORDER_CODE;
    orderCode = "";
    String cliCode = "";
  };
  cliDelOrderMsg(String ordercode, String clicode) {
    opCode = stringCommand.CLI_DELETE_ORDER_CODE;
    orderCode = ordercode;
    cliCode = clicode;
  };
  cliDelOrderMsg(String in) {  // Ricevuto: 9s01200113215410a00001
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    orderCode = in.substring(0,pos);  // Estraggo il codice del ordine
    in = in.substring(pos+1);
    pos = in.indexOf('');
    cliCode = in.substring(0,pos);  // Estraggo il codice del cliente
  };
  String toMsg(){ return Short.toString(opCode)+""+orderCode+""+cliCode+"\n"; };
  String getOrderCode(){ return orderCode;};  // Restituisce il codice del ordine
  String getCliCode()  { return cliCode;  };  // Restituisce il codice del cliente
  void setOrderCode(String ordercode){ orderCode = ordercode; };// Restituisce il codice del ordine
  void setCliCode(String clicode)    { cliCode = clicode; };    // Restituisce il codice del cliente
}

/************************  CLASSI PER LA GESTIONE DEI SERVER ***************************/
/** Classe astratta che specifica le caratteristiche comuni dei pacchetti scambiati fra server e server.
 *  Funge inoltre da classe radice per tutti i pacchetti usati nei dialoghi server con server */
abstract class servMsg extends Msg {
  String servCode;
  public servMsg() {};
  String getServCode() { return servCode; }
  void setServCode(String code) { servCode = code; };
  String getServCode(String in) {
    String app = in.substring(in.indexOf('')+1); // Elimino l'opCode
    return app.substring(0,app.indexOf(''));
  };
}

/** Pacchetto inviato dal server corrente subito dopo la connessione al next server.
 *  Consente di informare il next server del proprio stato e, se necessario, di innescare la procedura di aggiornamento delle informazioni  */
class servNewNextServerMsg extends servMsg {
  String Ip; // Indirizzo del nuovo Next Server
  int Port;  // Porta del nuovo Next Server
  long CL;   // Clock Logico del sistema

  servNewNextServerMsg() {
    opCode = stringCommand.SERV_NEW_NEXT_SERVER_CONNECT_CODE;
    servCode = "";
    Ip = "";
    Port = 0;
    CL = 0;
  };
  servNewNextServerMsg(String servcode, String ip, int port, long cl) {
    opCode = stringCommand.SERV_NEW_NEXT_SERVER_CONNECT_CODE;
    servCode = servcode;
    Ip = ip;
    Port = port;
    CL = cl;
  };
  servNewNextServerMsg(String in) {
    int pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);            // Estraggo il codice del Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    Ip = in.substring(0,pos);            // Estraggo l'Ip del nuovo Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    Port = Integer.parseInt(in.substring(0,pos)); // Estraggo la porta del nuovo Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    CL = Long.parseLong(in.substring(0,pos)); // Estraggo il Clock Logico
  };
  String toMsg(){ return Short.toString(opCode)+""+servCode+""+Ip+""+Integer.toString(Port)+""+Long.toString(CL)+"\n"; };
  short getOpCode(){return opCode;};// Restituisce il codice della operazione
  long getCl()  { return CL; };   // Restituisce il valore del Clock Logico
  String getIp(){ return Ip; };   // Restituisce il valore dell'Ip
  int getPort() { return Port; }; // Restituisce il valore della Porta
  void setOpCode(short opcode){opCode = opcode;};// Imposta il codice della operazione
  void setCl(long cl)   { CL = cl; };            // Imposta il valore del Clock Logico
  void setIp(String ip) { Ip = ip; };            // Imposta il valore dell'Ip
  void setPort(int port){ Port = port; }; // Imposta il valore della Porta
}

class servNewPrevServerMsg extends servMsg {
  String Ip; // Indirizzo del nuovo Next Server
  int Port;  // Porta del nuovo Next Server
  long CL;   // Clock Logico del sistema

  servNewPrevServerMsg() {
    opCode = stringCommand.SERV_NEW_PREV_SERVER_CONNECT_CODE;
    servCode = "";
    Ip = "";
    Port = 0;
    CL = 0;
  };
  servNewPrevServerMsg(String servcode, String ip, int port, long cl) {
    opCode = stringCommand.SERV_NEW_PREV_SERVER_CONNECT_CODE;
    servCode = servcode;
    Ip = ip;
    Port = port;
    CL = cl;
  };
  servNewPrevServerMsg(String in) {
    int pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);            // Estraggo il codice del Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    Ip = in.substring(0,pos);            // Estraggo l'Ip del nuovo Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    Port = Integer.parseInt(in.substring(0,pos)); // Estraggo la porta del nuovo Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    CL = Long.parseLong(in.substring(0,pos)); // Estraggo il Clock Logico
  };
  String toMsg(){ return Short.toString(opCode)+""+servCode+""+Ip+""+Integer.toString(Port)+""+Long.toString(CL)+"\n"; };
  short getOpCode(){return opCode;};// Restituisce il codice della operazione
  long getCl()  { return CL; };   // Restituisce il valore del Clock Logico
  String getIp(){ return Ip; };   // Restituisce il valore dell'Ip
  int getPort() { return Port; }; // Restituisce il valore della Porta
  void setOpCode(short opcode){opCode = opcode;};// Imposta il codice della operazione
  void setCl(long cl)   { CL = cl; };            // Imposta il valore del Clock Logico
  void setIp(String ip) { Ip = ip; };            // Imposta il valore dell'Ip
  void setPort(int port){ Port = port; }; // Imposta il valore della Porta
}
/*
* Acknowledge alla connessione,  usata dal server per fornire al server l'accettazione e le
 *  informazioni sullo stato attuale del sistema. Qualora queste siano diverse dalle attuali il
 *  Server forza il client ad una rilettura dello stato attraverso gli appositi pacchetti *
class servAckMsg extends servMsg {
  long CL;        // Clock Logico del sistema
  String servCode; // Codice del cliente
  ArrayList info; // Informazioni per gli utenti
  servAckMsg(){ opCode = stringCommand.CLI_ACK_CODE;  CL = 0;  servCode = ""; info = new ArrayList(); };
  servAckMsg(long cl,String servcode) {
    opCode = stringCommand.SERV_ACK_CODE;
    CL = cl;
    servCode = servcode;
    info = new ArrayList();
  };
  servAckMsg(long cl,String servcode,ArrayList inf) {
    opCode = stringCommand.SERV_ACK_CODE;
    CL = cl;
    servCode = servcode;
    info = inf;
  };
  servAckMsg(String in) {
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    CL = Integer.parseInt(in.substring(0,pos));     // Estraggo il Clock Logico
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);                 // Estraggo il Server Code
    in = in.substring(pos+1);
    while (!(in.compareTo("") ==0)){               // Se ho esaurito l'input
      pos = in.indexOf('');
      info.add(new msgInfo(in.substring(0,pos)));   // Estraggo le info
      in = in.substring(pos+1);
    };
  };
  String toMsg(){
    String app = Short.toString(opCode)+""+Long.toString(CL)+""+servCode+"";
    if (info!=null) {
      for (int i=0;i<info.size();i++){ app = app+((msgInfo)info.get(i)).toMsg()+""; };
    };
    return app+"\n";
  };
  String getServCode() { return servCode; };  // Restituisce il codice del server
  short getOpCode() { return opCode; };     // Restituisce il codice della operazione
  long getCl() { return CL; };              // Restituisce il valore del Clock Logico
  void addInfo(String i1,String i2) { info.add(new msgInfo(i1,i2)); }; // Aggiunge una info alla lista delle informazioni
  void addInfo(String i) { info.add(new msgInfo(i)); }; // Aggiunge una info alla lista delle informazioni
}*/
/** Messaggio utilizzato per verificare lo stato dell'anello dei server. */
class servTestMsg extends servMsg {
  ArrayList servInfo;
  servTestMsg() {
    opCode = stringCommand.SERV_TEST_CODE;
    servCode = "";
    servInfo = new ArrayList();
  };
  servTestMsg(String servcode,ArrayList servinfo) {
    opCode = stringCommand.SERV_TEST_CODE;
    servCode = servcode;
    if (servinfo == null)  servInfo = new ArrayList();
    else servInfo = servinfo;
  };
  servTestMsg(String in) {
    servInfo = new ArrayList();
    int pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);            // Estraggo il codice del Server
    in = in.substring(pos+1);
    while (!(in.compareTo("") ==0)){               // Se ho esaurito l'input
      pos = in.indexOf('');
      servInfo.add(in.substring(0,pos));   // Estraggo le info
      in = in.substring(pos+1);
    };
  };
  String toMsg(){
    String app = Short.toString(opCode)+""+servCode+"";
    if (servInfo!=null) {
      for (int i=0;i<servInfo.size();i++){ app = app+getInfo(i)+""; };
    };
    return app+"\n";
  };
  String getServCode() { return servCode; };  // Restituisce il codice del server che ha generato il pacchetto
  short getOpCode() { return opCode; };     // Restituisce il codice della operazione
  void addInfo(String in) { servInfo.add(in); }; // Aggiunge una info alla lista delle informazioni
  int getNumInfo() { return servInfo.size(); };
  String getInfo(int i) { return ((String) servInfo.get(i)); };
}

class servSendOrderMsg extends servMsg {
  String orderCode;      // Codice dell' ordine che si st inoltrando
  int weight;            // Peso del ordine, nesessario in caso di conflitto
  cliSendOrderMsg order; // Dati sull'ordine del cliente

  servSendOrderMsg(){
    opCode = stringCommand.SERV_SEND_ORDER_CODE;
    servCode = "";
    orderCode = "";
    weight = 0;
    order = new cliSendOrderMsg();
  };
  servSendOrderMsg(String servcode, String ordercode,int wght, cliSendOrderMsg clisendordermsg){
    opCode = stringCommand.SERV_SEND_ORDER_CODE;
    servCode = servcode;
    orderCode = ordercode;
    weight = wght;
    if (clisendordermsg == null) order = new cliSendOrderMsg();
    else order = clisendordermsg;
  };
  servSendOrderMsg(String in){
    int pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos));
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);
    in = in.substring(pos+1);
    pos = in.indexOf('');
    orderCode = in.substring(0,pos);
    in = in.substring(pos+1);
    pos = in.indexOf('');
    weight = Integer.parseInt(in.substring(0,pos));
    in = in.substring(pos+1);
    order = new cliSendOrderMsg(in);
  };
  void setWeight(int wght) { weight = wght; };
  int getWeight() { return weight; };
  void setOrderCode(String code) { orderCode = code; };
  String getOrderCode() { return orderCode; };
  String toMsg() {
   return Short.toString(opCode)+""+servCode+""+orderCode+""+Integer.toString(weight)+""+order.toMsg();
  };

}

class servDelOrderMsg extends servMsg {
  String orderCode;
  String cliCode;
  servDelOrderMsg() {
    opCode = stringCommand.SERV_DELETE_ORDER_CODE;
    servCode = "";
    orderCode = "";
    cliCode = "";
  };
  servDelOrderMsg(String servcode, String ordercode, String clicode) {
    opCode = stringCommand.SERV_DELETE_ORDER_CODE;
    servCode = servcode;
    orderCode = ordercode;
    cliCode = clicode;
  };
  servDelOrderMsg(String in) {
    int pos;
    pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);  // Estraggo il codice del server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    orderCode = in.substring(0,pos);  // Estraggo il codice del ordine
    in = in.substring(pos+1);
    pos = in.indexOf('');
    cliCode = in.substring(0,pos);  // Estraggo il codice del cliente
  };
  String toMsg(){ return Short.toString(opCode)+""+servCode+""+orderCode+""+cliCode+"\n"; };
  String getOrderCode(){ return orderCode;};  // Restituisce il codice del ordine
  String getCliCode()  { return cliCode;  };  // Restituisce il codice del cliente
  String getServCode() { return servCode; };  // Restituisce il codice del server
  void setOrderCode(String ordercode){ orderCode = ordercode; };// Imposta il codice del ordine
  void setCliCode(String clicode)    { cliCode = clicode; };    // Imposta il codice del cliente
  void setServCode(String servcode)    { servCode = servcode; };// Imposta il codice del server
}

/** Pacchetto inviato dal server corrente subito dopo la connessione al next server.
 *  Consente di informare il next server del proprio stato e, se necessario, di innescare la procedura di aggiornamento delle informazioni  */
/*
 class servNewNextServerMsg extends servMsg {
  String Ip; // Indirizzo del nuovo Next Server
  int Port;  // Porta del nuovo Next Server
  long CL;   // Clock Logico del sistema

  servNewNextServerMsg() {
    opCode = stringCommand.SERV_NEW_NEXT_SERVER_CONNECT_CODE;
    servCode = "";
    Ip = "";
    Port = 0;
    CL = 0;
  };
  servNewNextServerMsg(String servcode, String ip, int port, long cl) {
    opCode = stringCommand.SERV_NEW_NEXT_SERVER_CONNECT_CODE;
    servCode = servcode;
    Ip = ip;
    Port = port;
    CL = cl;
  };
  servNewNextServerMsg(String in) {
    int pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);            // Estraggo il codice del Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    Ip = in.substring(0,pos);            // Estraggo l'Ip del nuovo Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    Port = Integer.parseInt(in.substring(0,pos)); // Estraggo la porta del nuovo Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    CL = Long.parseLong(in.substring(0,pos)); // Estraggo il Clock Logico
  };
  String toMsg(){ return Short.toString(opCode)+""+servCode+""+Ip+""+Integer.toString(Port)+""+Long.toString(CL)+"\n"; };
  short getOpCode(){return opCode;};// Restituisce il codice della operazione
  long getCl()  { return CL; };   // Restituisce il valore del Clock Logico
  String getIp(){ return Ip; };   // Restituisce il valore dell'Ip
  int getPort() { return Port; }; // Restituisce il valore della Porta
  void setOpCode(short opcode){opCode = opcode;};// Imposta il codice della operazione
  void setCl(long cl)   { CL = cl; };            // Imposta il valore del Clock Logico
  void setIp(String ip) { Ip = ip; };            // Imposta il valore dell'Ip
  void setPort(int port){ Port = port; }; // Imposta il valore della Porta
}
*/
class servCrashServerMsg extends servMsg {
  String Ip; // Indirizzo del nuovo Next Server
  int Port;  // Porta del nuovo Next Server
  long CL;   // Clock Logico del sistema
  ArrayList info; // codici dei server ancora attivi
  servCrashServerMsg() {
    opCode = stringCommand.SERV_CRASH_SERVER_CODE;
    servCode = "";
    Ip = "";
    Port = 0;
    CL = 0;
    info = new ArrayList();
  };
  servCrashServerMsg(String servcode, String ip, int port, long cl, ArrayList inf) {
    opCode = stringCommand.SERV_CRASH_SERVER_CODE;
    servCode = servcode;
    Ip = ip;
    Port = port;
    CL = cl;
    if(inf==null) info = new ArrayList();
    else info = inf;
  };
  servCrashServerMsg(String in) {
    info = new ArrayList();
    int pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);            // Estraggo il codice del Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    Ip = in.substring(0,pos);            // Estraggo l'Ip del nuovo Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    Port = Integer.parseInt(in.substring(0,pos)); // Estraggo la porta del nuovo Server
    in = in.substring(pos+1);
    pos = in.indexOf('');
    CL = Long.parseLong(in.substring(0,pos)); // Estraggo il Clock Logico
    in = in.substring(pos+1);
    while (!(in.compareTo("")==0)){               // Se ho esaurito l'input
      pos = in.indexOf('');
      info.add(in.substring(0,pos));   // Estraggo le info
      in = in.substring(pos+1);
    };
  };
  String toMsg(){
    String app = Short.toString(opCode)+""+servCode+""+Ip+""+Integer.toString(Port)+""+Long.toString(CL)+"";
    for (int i=0;i<info.size();i++){ app = app+getInfo(i)+""; };
    return app+"\n";
  };
  short getOpCode(){return opCode;};// Restituisce il codice della operazione
  long getCl()  { return CL; };   // Restituisce il valore del Clock Logico
  String getIp(){ return Ip; };   // Restituisce il valore dell'Ip
  int getPort() { return Port; }; // Restituisce il valore della Porta
  void setOpCode(short opcode){opCode = opcode;};// Imposta il codice della operazione
  void setCl(long cl)   { CL = cl; };            // Imposta il valore del Clock Logico
  void setIp(String ip) { Ip = ip; };            // Imposta il valore dell'Ip
  void setPort(int port){ Port = port; }; // Imposta il valore della Porta
  String getInfo(int i){ return (String)info.get(i);};
  int getNumInfo() { return info.size(); };
  void addInfo(String inf) { info.add(inf); };
  ArrayList getAllInfo() { return info; };
}

class servRecoveryCompletedMsg extends servMsg {
  ArrayList info; // codici dei server ancora attivi
  servRecoveryCompletedMsg() {
    opCode = stringCommand.SERV_RECOVERY_COMPLETED_CODE;
    servCode = "";
    info = new ArrayList();
  };
  servRecoveryCompletedMsg(String servcode, ArrayList inf) {
    opCode = stringCommand.SERV_RECOVERY_COMPLETED_CODE;
    servCode = servcode;
    if(inf==null) info = new ArrayList();
    else info = inf;
  };
  servRecoveryCompletedMsg(String in) {
    info = new ArrayList();
    int pos = in.indexOf('');
    opCode = Short.parseShort(in.substring(0,pos)); // Estraggo l'opCode
    in = in.substring(pos+1);
    pos = in.indexOf('');
    servCode = in.substring(0,pos);            // Estraggo il codice del Server
    in = in.substring(pos+1);
    while (!(in.compareTo("")==0)){               // Se ho esaurito l'input
      pos = in.indexOf('');
      info.add(in.substring(0,pos));   // Estraggo le info
      in = in.substring(pos+1);
    };
  };
  String toMsg(){
    String app = Short.toString(opCode)+""+servCode+"";
    for (int i=0;i<info.size();i++){ app = app+getInfo(i)+""; };
    return app+"\n";
  };
  short getOpCode(){return opCode;};// Restituisce il codice della operazione
  void setOpCode(short opcode){opCode = opcode;};// Imposta il codice della operazione
  String getInfo(int i){ return (String)info.get(i);};
  int getNumInfo() { return info.size(); };
  void addInfo(String inf) { info.add(inf); };
  ArrayList getAllInfo() { return info; };
}
/*********************************  CLASSI DI SUPPORTO *********************************/
class msgInfo {
  String arg1;
  String arg2;
  msgInfo() { arg1=new String();    arg2=new String(); }
  msgInfo(String a) {
    int pos = a.indexOf('=');     // Ricerco il separatore e
    arg1 = a.substring(0,pos);  // estraggo le info
    arg2 = a.substring(pos+1);
  };
  msgInfo(String a,char s) { // s  il separatore
    int pos = a.indexOf(s);     // Ricerco il separatore e
    arg1 = a.substring(0,pos);  // estraggo le info
    arg2 = a.substring(pos+1);
  };

  msgInfo(String a1,String a2) { arg1=a1;   arg2=a2; }
  String get(int arg) { if (arg==1){ return arg1; }else{ return arg2; }; };
  void set(int arg,String str) { if (arg==1){ arg1=str; }else{ arg2=str; }; };
  String toMsg() { return arg1+"="+arg2; }
};